@@ -180,6 +180,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 
     ProStringList &bundledFiles = project->values("QMAKE_BUNDLED_FILES");
 
+    writeExportedVariables(t);
+
     t << "####### Compiler, tools and options\n\n";
     t << "CC            = " << var("QMAKE_CC") << endl;
     t << "CXX           = " << var("QMAKE_CXX") << endl;
@@ -405,7 +407,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
         const ProStringList &l = project->values("QMAKE_PRL_INTERNAL_FILES");
         ProStringList::ConstIterator it;
         for(it = l.begin(); it != l.end(); ++it) {
-            QMakeMetaInfo libinfo(project);
+            QMakeMetaInfo libinfo;
             if (libinfo.readLib((*it).toQString()) && !libinfo.isEmpty("QMAKE_PRL_BUILD_DIR")) {
                 ProString dir;
                 int slsh = (*it).lastIndexOf(Option::dir_sep);
