@@ -153,8 +153,6 @@
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qpointer.h>
 
-#ifndef QT_NO_ANIMATION
-
 #define DEFAULT_TIMER_INTERVAL 16
 #define PAUSE_TIMER_COARSE_THRESHOLD 2000
 
@@ -215,9 +213,7 @@ typedef QList<QAbstractAnimation*>::ConstIterator AnimationListConstIt;
     QUnifiedTimer drives animations indirectly, via QAbstractAnimationTimer.
 */
 
-#ifndef QT_NO_THREAD
 Q_GLOBAL_STATIC(QThreadStorage<QUnifiedTimer *>, unifiedTimer)
-#endif
 
 QUnifiedTimer::QUnifiedTimer() :
     QObject(), defaultDriver(this), lastTick(0), timingInterval(DEFAULT_TIMER_INTERVAL),
@@ -234,18 +230,12 @@ QUnifiedTimer::QUnifiedTimer() :
 QUnifiedTimer *QUnifiedTimer::instance(bool create)
 {
     QUnifiedTimer *inst;
-#ifndef QT_NO_THREAD
     if (create && !unifiedTimer()->hasLocalData()) {
         inst = new QUnifiedTimer;
         unifiedTimer()->setLocalData(inst);
     } else {
         inst = unifiedTimer() ? unifiedTimer()->localData() : 0;
     }
-#else
-    Q_UNUSED(create);
-    static QUnifiedTimer unifiedTimer;
-    inst = &unifiedTimer;
-#endif
     return inst;
 }
 
@@ -554,7 +544,7 @@ bool QUnifiedTimer::canUninstallAnimationDriver(QAnimationDriver *d)
     return d == driver && driver != &defaultDriver;
 }
 
-#ifndef QT_NO_THREAD
+#if QT_CONFIG(thread)
 Q_GLOBAL_STATIC(QThreadStorage<QAnimationTimer *>, animationTimer)
 #endif
 
@@ -569,7 +559,7 @@ QAnimationTimer::QAnimationTimer() :
 QAnimationTimer *QAnimationTimer::instance(bool create)
 {
     QAnimationTimer *inst;
-#ifndef QT_NO_THREAD
+#if QT_CONFIG(thread)
     if (create && !animationTimer()->hasLocalData()) {
         inst = new QAnimationTimer;
         animationTimer()->setLocalData(inst);
@@ -1093,7 +1083,7 @@ QAbstractAnimation::State QAbstractAnimation::state() const
 
 /*!
     If this animation is part of a QAnimationGroup, this function returns a
-    pointer to the group; otherwise, it returns 0.
+    pointer to the group; otherwise, it returns \nullptr.
 
     \sa QAnimationGroup::addAnimation()
 */
@@ -1485,5 +1475,3 @@ QT_END_NAMESPACE
 
 #include "moc_qabstractanimation.cpp"
 #include "moc_qabstractanimation_p.cpp"
-
-#endif //QT_NO_ANIMATION
