@@ -2,7 +2,7 @@
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
 ** Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
-** Copyright (C) 2016 Intel Corporation.
+** Copyright (C) 2018 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -41,6 +41,7 @@
 
 #include "qglobal_p.h"
 #include "qlogging.h"
+#include "qlogging_p.h"
 #include "qlist.h"
 #include "qbytearray.h"
 #include "qstring.h"
@@ -56,18 +57,29 @@
 #include "private/qloggingregistry_p.h"
 #include "private/qcoreapplication_p.h"
 #include "private/qsimd_p.h"
+#include <qtcore_tracepoints_p.h>
 #endif
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
 #endif
+#ifdef Q_CC_MSVC
+#include <intrin.h>
+#endif
 #if QT_CONFIG(slog2)
-#include <slog2.h>
+#include <sys/slog2.h>
+#endif
+#if QT_HAS_INCLUDE(<paths.h>)
+#include <paths.h>
 #endif
 
 #ifdef Q_OS_ANDROID
 #include <android/log.h>
 #endif
 
+#ifdef Q_OS_DARWIN
+#include <QtCore/private/qcore_mac_p.h>
+#endif
+
 #if QT_CONFIG(journald)
 # define SD_JOURNAL_SUPPRESS_LOCATION
 # include <systemd/sd-journal.h>
@@ -83,8 +95,11 @@
 # include "private/qcore_unix_p.h"
 #endif
 
-#ifndef QT_BOOTSTRAPPED
-#if !defined QT_NO_REGULAREXPRESSION
+#ifdef Q_OS_WASM
+#include <emscripten/emscripten.h>
+#endif
+
+#if QT_CONFIG(regularexpression)
 #  ifdef __UCLIBC__
 #    if __UCLIBC_HAS_BACKTRACE__
 #      define QLOGGING_HAVE_BACKTRACE
@@ -98,6 +113,7 @@
 extern char *__progname;
 #endif
 
+#ifndef QT_BOOTSTRAPPED
 #if defined(Q_OS_LINUX) && (defined(__GLIBC__) || QT_HAS_INCLUDE(<sys/syscall.h>))
 #  include <sys/syscall.h>
 
@@ -152,6 +168,7 @@ Q_NORETURN
 #endif
 static void qt_message_fatal(QtMsgType, const QMessageLogContext &context, const QString &message);
 static void qt_message_print(QtMsgType, const QMessageLogContext &context, const QString &message);
+static void qt_message_print(const QString &message);
 
 static int checked_var_value(const char *varname)
 {
@@ -191,55 +208,110 @@ static bool isFatal(QtMsgType msgType)
     return false;
 }
 
-static bool willLogToConsole()
+static bool isDefaultCategory(const char *category)
+{
+    return !category || strcmp(category, "default") == 0;
+}
+
+/*!
+    Returns true if writing to \c stderr is supported.
+
+    \internal
+    \sa stderrHasConsoleAttached()
+*/
+static bool systemHasStderr()
 {
 #if defined(Q_OS_WINRT)
-    // these systems have no stderr, so always log to the system log
-    return false;
-#elif defined(QT_BOOTSTRAPPED)
-    return true;
-#else
-    // rules to determine if we'll log preferably to the console:
-    //  1) if QT_LOGGING_TO_CONSOLE is set, it determines behavior:
-    //    - if it's set to 0, we will not log to console
-    //    - if it's set to 1, we will log to console
-    //  2) otherwise, we will log to console if we have a console window (Windows)
-    //     or a controlling TTY (Unix). This is done even if stderr was redirected
-    //     to the blackhole device (NUL or /dev/null).
-
-    bool ok = true;
-    uint envcontrol = qgetenv("QT_LOGGING_TO_CONSOLE").toUInt(&ok);
-    if (ok)
-        return envcontrol;
-
-#  ifdef Q_OS_WIN
-    return GetConsoleWindow();
-#  elif defined(Q_OS_UNIX)
-    // if /dev/tty exists, we can only open it if we have a controlling TTY
-    int devtty = qt_safe_open("/dev/tty", O_RDONLY);
-    if (devtty == -1 && (errno == ENOENT || errno == EPERM || errno == ENXIO)) {
-        // no /dev/tty, fall back to isatty on stderr
-        return isatty(STDERR_FILENO);
-    } else if (devtty != -1) {
-        // there is a /dev/tty and we could open it: we have a controlling TTY
-        qt_safe_close(devtty);
-        return true;
-    }
-
-    // no controlling TTY
-    return false;
-#  else
-#    error "Not Unix and not Windows?"
-#  endif
+    return false; // WinRT has no stderr
 #endif
+
+    return true;
 }
 
-Q_CORE_EXPORT bool qt_logging_to_console()
+/*!
+    Returns true if writing to \c stderr will end up in a console/terminal visible to the user.
+
+    This is typically the case if the application was started from the command line.
+
+    If the application is started without a controlling console/terminal, but the parent
+    process reads \c stderr and presents it to the user in some other way, the parent process
+    may override the detection in this function by setting the QT_ASSUME_STDERR_HAS_CONSOLE
+    environment variable to \c 1.
+
+    \note Qt Creator does not implement a pseudo TTY, nor does it launch apps with
+    the override environment variable set, but it will read stderr and print it to
+    the user, so in effect this function can not be used to conclude that stderr
+    output will _not_ be visible to the user, as even if this function returns false,
+    the output might still end up visible to the user. For this reason, we don't guard
+    the stderr output in the default message handler with stderrHasConsoleAttached().
+
+    \internal
+    \sa systemHasStderr()
+*/
+static bool stderrHasConsoleAttached()
 {
-    static const bool logToConsole = willLogToConsole();
-    return logToConsole;
+    static const bool stderrHasConsoleAttached = []() -> bool {
+        if (!systemHasStderr())
+            return false;
+
+        if (qEnvironmentVariableIntValue("QT_LOGGING_TO_CONSOLE")) {
+            fprintf(stderr, "warning: Environment variable QT_LOGGING_TO_CONSOLE is deprecated, use\n"
+                            "QT_ASSUME_STDERR_HAS_CONSOLE and/or QT_FORCE_STDERR_LOGGING instead.\n");
+            return true;
+        }
+
+        if (qEnvironmentVariableIntValue("QT_ASSUME_STDERR_HAS_CONSOLE"))
+            return true;
+
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINRT)
+        return GetConsoleWindow();
+#elif defined(Q_OS_UNIX)
+#       ifndef _PATH_TTY
+#       define _PATH_TTY "/dev/tty"
+#       endif
+
+        // If we can open /dev/tty, we have a controlling TTY
+        int ttyDevice = -1;
+        if ((ttyDevice = qt_safe_open(_PATH_TTY, O_RDONLY)) >= 0) {
+            qt_safe_close(ttyDevice);
+            return true;
+        } else if (errno == ENOENT || errno == EPERM || errno == ENXIO) {
+            // Fall back to isatty for some non-critical errors
+            return isatty(STDERR_FILENO);
+        } else {
+            return false;
+        }
+#else
+        return false; // No way to detect if stderr has a console attached
+#endif
+    }();
+
+    return stderrHasConsoleAttached;
 }
 
+
+namespace QtPrivate {
+
+/*!
+    Returns true if logging \c stderr should be ensured.
+
+    This is normally the case if \c stderr has a console attached, but may be overridden
+    by the user by setting the QT_FORCE_STDERR_LOGGING environment variable to \c 1.
+
+    \internal
+    \sa stderrHasConsoleAttached()
+*/
+bool shouldLogToStderr()
+{
+    static bool forceStderrLogging = qEnvironmentVariableIntValue("QT_FORCE_STDERR_LOGGING");
+    return forceStderrLogging || stderrHasConsoleAttached();
+}
+
+
+} // QtPrivate
+
+using namespace QtPrivate;
+
 /*!
     \class QMessageLogContext
     \inmodule QtCore
@@ -1144,8 +1216,8 @@ void QMessagePattern::setPattern(const QString &pattern)
                 tokens[i] = backtraceTokenC;
                 QString backtraceSeparator = QStringLiteral("|");
                 int backtraceDepth = 5;
-                QRegularExpression depthRx(QStringLiteral(" depth=(?|\"([^\"]*)\"|([^ }]*))"));
-                QRegularExpression separatorRx(QStringLiteral(" separator=(?|\"([^\"]*)\"|([^ }]*))"));
+                static const QRegularExpression depthRx(QStringLiteral(" depth=(?|\"([^\"]*)\"|([^ }]*))"));
+                static const QRegularExpression separatorRx(QStringLiteral(" separator=(?|\"([^\"]*)\"|([^ }]*))"));
                 QRegularExpressionMatch m = depthRx.match(lexeme);
                 if (m.hasMatch()) {
                     int depth = m.capturedRef(1).toInt();
@@ -1203,20 +1275,10 @@ void QMessagePattern::setPattern(const QString &pattern)
         error += QLatin1String("QT_MESSAGE_PATTERN: %{if-*} cannot be nested\n");
     else if (inIf)
         error += QLatin1String("QT_MESSAGE_PATTERN: missing %{endif}\n");
-    if (!error.isEmpty()) {
-#if defined(Q_OS_WINRT)
-        OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
-        if (0)
-#elif defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
-        if (!qt_logging_to_console()) {
-            OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
-        } else
-#endif
-        {
-            fprintf(stderr, "%s", error.toLocal8Bit().constData());
-            fflush(stderr);
-        }
-    }
+
+    if (!error.isEmpty())
+        qt_message_print(error);
+
     literals = new const char*[literalsVar.size() + 1];
     literals[literalsVar.size()] = 0;
     memcpy(literals, literalsVar.constData(), literalsVar.size() * sizeof(const char*));
@@ -1241,10 +1303,9 @@ static QStringList backtraceFramesForLogMessage(int frameCount)
     // The offset and function name are optional.
     // This regexp tries to extract the library name (without the path) and the function name.
     // This code is protected by QMessagePattern::mutex so it is thread safe on all compilers
-    static QRegularExpression rx(QStringLiteral("^(?:[^(]*/)?([^(/]+)\\(([^+]*)(?:[\\+[a-f0-9x]*)?\\) \\[[a-f0-9x]*\\]$"),
-                                 QRegularExpression::OptimizeOnFirstUsageOption);
+    static const QRegularExpression rx(QStringLiteral("^(?:[^(]*/)?([^(/]+)\\(([^+]*)(?:[\\+[a-f0-9x]*)?\\) \\[[a-f0-9x]*\\]$"));
 
-    QVarLengthArray<void*, 32> buffer(7 + frameCount);
+    QVarLengthArray<void*, 32> buffer(8 + frameCount);
     int n = backtrace(buffer.data(), buffer.size());
     if (n > 0) {
         int numberPrinted = 0;
@@ -1306,57 +1367,6 @@ static QString formatBacktraceForLogMessage(const QMessagePattern::BacktracePara
 }
 #endif // QLOGGING_HAVE_BACKTRACE && !QT_BOOTSTRAPPED
 
-#if QT_CONFIG(slog2)
-#ifndef QT_LOG_CODE
-#define QT_LOG_CODE 9000
-#endif
-
-static void slog2_default_handler(QtMsgType msgType, const char *message)
-{
-    if (slog2_set_default_buffer((slog2_buffer_t)-1) == 0) {
-        slog2_buffer_set_config_t buffer_config;
-        slog2_buffer_t buffer_handle;
-
-        buffer_config.buffer_set_name = __progname;
-        buffer_config.num_buffers = 1;
-        buffer_config.verbosity_level = SLOG2_DEBUG1;
-        buffer_config.buffer_config[0].buffer_name = "default";
-        buffer_config.buffer_config[0].num_pages = 8;
-
-        if (slog2_register(&buffer_config, &buffer_handle, 0) == -1) {
-            fprintf(stderr, "Error registering slogger2 buffer!\n");
-            fprintf(stderr, "%s", message);
-            fflush(stderr);
-            return;
-        }
-
-        // Set as the default buffer
-        slog2_set_default_buffer(buffer_handle);
-    }
-    int severity;
-    //Determines the severity level
-    switch (msgType) {
-    case QtDebugMsg:
-        severity = SLOG2_DEBUG1;
-        break;
-    case QtInfoMsg:
-        severity = SLOG2_INFO;
-        break;
-    case QtWarningMsg:
-        severity = SLOG2_NOTICE;
-        break;
-    case QtCriticalMsg:
-        severity = SLOG2_WARNING;
-        break;
-    case QtFatalMsg:
-        severity = SLOG2_ERROR;
-        break;
-    }
-    //writes to the slog2 buffer
-    slog2c(NULL, QT_LOG_CODE, severity, message);
-}
-#endif // slog2
-
 Q_GLOBAL_STATIC(QMessagePattern, qMessagePattern)
 
 /*!
@@ -1474,7 +1484,7 @@ QString qFormatLogMessage(QtMsgType type, const QMessageLogContext &context, con
             }
 #endif // !QT_BOOTSTRAPPED
         } else if (token == ifCategoryTokenC) {
-            if (!context.category || (strcmp(context.category, "default") == 0))
+            if (isDefaultCategory(context.category))
                 skip = true;
 #define HANDLE_IF_TOKEN(LEVEL)  \
         } else if (token == if##LEVEL##TokenC) { \
@@ -1506,11 +1516,80 @@ static QBasicAtomicPointer<void (QtMsgType, const char*)> msgHandler = Q_BASIC_A
 // pointer to QtMessageHandler debug handler (with context)
 static QBasicAtomicPointer<void (QtMsgType, const QMessageLogContext &, const QString &)> messageHandler = Q_BASIC_ATOMIC_INITIALIZER(qDefaultMessageHandler);
 
+// ------------------------ Alternate logging sinks -------------------------
+
+#if defined(QT_BOOTSTRAPPED)
+    // Boostrapped tools always print to stderr, so no need for alternate sinks
+#else
+
+#if QT_CONFIG(slog2)
+#ifndef QT_LOG_CODE
+#define QT_LOG_CODE 9000
+#endif
+
+static bool slog2_default_handler(QtMsgType type, const QMessageLogContext &context, const QString &message)
+{
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+    formattedMessage.append(QLatin1Char('\n'));
+    if (slog2_set_default_buffer((slog2_buffer_t)-1) == 0) {
+        slog2_buffer_set_config_t buffer_config;
+        slog2_buffer_t buffer_handle;
+
+        buffer_config.buffer_set_name = __progname;
+        buffer_config.num_buffers = 1;
+        buffer_config.verbosity_level = SLOG2_DEBUG1;
+        buffer_config.buffer_config[0].buffer_name = "default";
+        buffer_config.buffer_config[0].num_pages = 8;
+
+        if (slog2_register(&buffer_config, &buffer_handle, 0) == -1) {
+            fprintf(stderr, "Error registering slogger2 buffer!\n");
+            fprintf(stderr, "%s", formattedMessage.toLocal8Bit().constData());
+            fflush(stderr);
+            return false;
+        }
+
+        // Set as the default buffer
+        slog2_set_default_buffer(buffer_handle);
+    }
+    int severity;
+    //Determines the severity level
+    switch (type) {
+    case QtDebugMsg:
+        severity = SLOG2_DEBUG1;
+        break;
+    case QtInfoMsg:
+        severity = SLOG2_INFO;
+        break;
+    case QtWarningMsg:
+        severity = SLOG2_NOTICE;
+        break;
+    case QtCriticalMsg:
+        severity = SLOG2_WARNING;
+        break;
+    case QtFatalMsg:
+        severity = SLOG2_ERROR;
+        break;
+    }
+    //writes to the slog2 buffer
+    slog2c(NULL, QT_LOG_CODE, severity, formattedMessage.toLocal8Bit().constData());
+
+    return true; // Prevent further output to stderr
+}
+#endif // slog2
+
 #if QT_CONFIG(journald)
-static void systemd_default_message_handler(QtMsgType type,
+static bool systemd_default_message_handler(QtMsgType type,
                                             const QMessageLogContext &context,
                                             const QString &message)
 {
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+
     int priority = LOG_INFO; // Informational
     switch (type) {
     case QtDebugMsg:
@@ -1530,19 +1609,26 @@ static void systemd_default_message_handler(QtMsgType type,
         break;
     }
 
-    sd_journal_send("MESSAGE=%s",     message.toUtf8().constData(),
+    sd_journal_send("MESSAGE=%s",     formattedMessage.toUtf8().constData(),
                     "PRIORITY=%i",    priority,
                     "CODE_FUNC=%s",   context.function ? context.function : "unknown",
                     "CODE_LINE=%d",   context.line,
                     "CODE_FILE=%s",   context.file ? context.file : "unknown",
                     "QT_CATEGORY=%s", context.category ? context.category : "unknown",
                     NULL);
+
+    return true; // Prevent further output to stderr
 }
 #endif
 
 #if QT_CONFIG(syslog)
-static void syslog_default_message_handler(QtMsgType type, const char *message)
+static bool syslog_default_message_handler(QtMsgType type, const QMessageLogContext &context, const QString &message)
 {
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+
     int priority = LOG_INFO; // Informational
     switch (type) {
     case QtDebugMsg:
@@ -1562,15 +1648,22 @@ static void syslog_default_message_handler(QtMsgType type, const char *message)
         break;
     }
 
-    syslog(priority, "%s", message);
+    syslog(priority, "%s", formattedMessage.toUtf8().constData());
+
+    return true; // Prevent further output to stderr
 }
 #endif
 
-#ifdef Q_OS_ANDROID
-static void android_default_message_handler(QtMsgType type,
+#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+static bool android_default_message_handler(QtMsgType type,
                                   const QMessageLogContext &context,
                                   const QString &message)
 {
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+
     android_LogPriority priority = ANDROID_LOG_DEBUG;
     switch (type) {
     case QtDebugMsg: priority = ANDROID_LOG_DEBUG; break;
@@ -1580,47 +1673,108 @@ static void android_default_message_handler(QtMsgType type,
     case QtFatalMsg: priority = ANDROID_LOG_FATAL; break;
     };
 
-    __android_log_print(priority, qPrintable(QCoreApplication::applicationName()),
-                        "%s:%d (%s): %s\n", context.file, context.line,
-                        context.function, qPrintable(message));
+    __android_log_print(priority, qPrintable(QCoreApplication::applicationName()), "%s\n", qPrintable(formattedMessage));
+
+    return true; // Prevent further output to stderr
 }
 #endif //Q_OS_ANDROID
 
+#ifdef Q_OS_WIN
+static bool win_message_handler(QtMsgType type, const QMessageLogContext &context, const QString &message)
+{
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+    formattedMessage.append(QLatin1Char('\n'));
+    OutputDebugString(reinterpret_cast<const wchar_t *>(formattedMessage.utf16()));
+
+    return true; // Prevent further output to stderr
+}
+#endif
+
+#ifdef Q_OS_WASM
+static bool wasm_default_message_handler(QtMsgType type,
+                                  const QMessageLogContext &context,
+                                  const QString &message)
+{
+    if (shouldLogToStderr())
+        return false; // Leave logging up to stderr handler
+
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+    int emOutputFlags = (EM_LOG_CONSOLE | EM_LOG_DEMANGLE);
+    QByteArray localMsg = message.toLocal8Bit();
+    switch (type) {
+    case QtDebugMsg:
+        break;
+    case QtInfoMsg:
+        break;
+    case QtWarningMsg:
+        emOutputFlags |= EM_LOG_WARN;
+        break;
+    case QtCriticalMsg:
+        emOutputFlags |= EM_LOG_ERROR;
+        break;
+    case QtFatalMsg:
+        emOutputFlags |= EM_LOG_ERROR;
+    }
+    emscripten_log(emOutputFlags, "%s\n", qPrintable(formattedMessage));
+
+    return true; // Prevent further output to stderr
+}
+#endif
+
+#endif // Bootstrap check
+
+// --------------------------------------------------------------------------
+
+static void stderr_message_handler(QtMsgType type, const QMessageLogContext &context, const QString &message)
+{
+    QString formattedMessage = qFormatLogMessage(type, context, message);
+
+    // print nothing if message pattern didn't apply / was empty.
+    // (still print empty lines, e.g. because message itself was empty)
+    if (formattedMessage.isNull())
+        return;
+
+    fprintf(stderr, "%s\n", formattedMessage.toLocal8Bit().constData());
+    fflush(stderr);
+}
+
 /*!
     \internal
 */
 static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &context,
-                                   const QString &buf)
+                                   const QString &message)
 {
-    QString logMessage = qFormatLogMessage(type, context, buf);
+    bool handledStderr = false;
 
-    // print nothing if message pattern didn't apply / was empty.
-    // (still print empty lines, e.g. because message itself was empty)
-    if (logMessage.isNull())
-        return;
+    // A message sink logs the message to a structured or unstructured destination,
+    // optionally formatting the message if the latter, and returns true if the sink
+    // handled stderr output as well, which will shortcut our default stderr output.
+    // In the future, if we allow multiple/dynamic sinks, this will be iterating
+    // a list of sinks.
 
-    if (!qt_logging_to_console()) {
-#if defined(Q_OS_WIN)
-        logMessage.append(QLatin1Char('\n'));
-        OutputDebugString(reinterpret_cast<const wchar_t *>(logMessage.utf16()));
-        return;
-#elif QT_CONFIG(slog2)
-        logMessage.append(QLatin1Char('\n'));
-        slog2_default_handler(type, logMessage.toLocal8Bit().constData());
-        return;
-#elif QT_CONFIG(journald)
-        systemd_default_message_handler(type, context, logMessage);
-        return;
-#elif QT_CONFIG(syslog)
-        syslog_default_message_handler(type, logMessage.toUtf8().constData());
-        return;
-#elif defined(Q_OS_ANDROID)
-        android_default_message_handler(type, context, logMessage);
-        return;
+#if !defined(QT_BOOTSTRAPPED)
+# if defined(Q_OS_WIN)
+    handledStderr |= win_message_handler(type, context, message);
+# elif QT_CONFIG(slog2)
+    handledStderr |= slog2_default_handler(type, context, message);
+# elif QT_CONFIG(journald)
+    handledStderr |= systemd_default_message_handler(type, context, message);
+# elif QT_CONFIG(syslog)
+    handledStderr |= syslog_default_message_handler(type, context, message);
+# elif defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+    handledStderr |= android_default_message_handler(type, context, message);
+# elif defined(QT_USE_APPLE_UNIFIED_LOGGING)
+    handledStderr |= AppleUnifiedLogger::messageHandler(type, context, message);
+# elif defined Q_OS_WASM
+    handledStderr |= wasm_default_message_handler(type, context, message);
+# endif
 #endif
-    }
-    fprintf(stderr, "%s\n", logMessage.toLocal8Bit().constData());
-    fflush(stderr);
+
+    if (!handledStderr)
+        stderr_message_handler(type, context, message);
 }
 
 /*!
@@ -1658,8 +1812,10 @@ static void ungrabMessageHandler() { }
 static void qt_message_print(QtMsgType msgType, const QMessageLogContext &context, const QString &message)
 {
 #ifndef QT_BOOTSTRAPPED
+    Q_TRACE(qt_message_print, msgType, context.category, context.function, context.file, context.line, message);
+
     // qDebug, qWarning, ... macros do not check whether category is enabled
-    if (!context.category || (strcmp(context.category, "default") == 0)) {
+    if (isDefaultCategory(context.category)) {
         if (QLoggingCategory *defaultCategory = QLoggingCategory::defaultCategory()) {
             if (!defaultCategory->isEnabled(msgType))
                 return;
@@ -1683,6 +1839,21 @@ static void qt_message_print(QtMsgType msgType, const QMessageLogContext &contex
     }
 }
 
+static void qt_message_print(const QString &message)
+{
+#if defined(Q_OS_WINRT)
+    OutputDebugString(reinterpret_cast<const wchar_t*>(message.utf16()));
+    return;
+#elif defined(Q_OS_WIN) && !defined(QT_BOOTSTRAPPED)
+    if (!shouldLogToStderr()) {
+        OutputDebugString(reinterpret_cast<const wchar_t*>(message.utf16()));
+        return;
+    }
+#endif
+    fprintf(stderr, "%s", message.toLocal8Bit().constData());
+    fflush(stderr);
+}
+
 static void qt_message_fatal(QtMsgType, const QMessageLogContext &context, const QString &message)
 {
 #if defined(Q_CC_MSVC) && defined(QT_DEBUG) && defined(_DEBUG) && defined(_CRT_ERROR)
@@ -1707,7 +1878,31 @@ static void qt_message_fatal(QtMsgType, const QMessageLogContext &context, const
     Q_UNUSED(message);
 #endif
 
+#ifdef Q_OS_WIN
+    // std::abort() in the MSVC runtime will call _exit(3) if the abort
+    // behavior is _WRITE_ABORT_MSG - see also _set_abort_behavior(). This is
+    // the default for a debug-mode build of the runtime. Worse, MinGW's
+    // std::abort() implementation (in msvcrt.dll) is basically a call to
+    // _exit(3) too. Unfortunately, _exit() and _Exit() *do* run the static
+    // destructors of objects in DLLs, a violation of the C++ standard (see
+    // [support.start.term]). So we bypass std::abort() and directly
+    // terminate the application.
+
+#  if defined(Q_CC_MSVC) && !defined(Q_CC_INTEL)
+    if (IsProcessorFeaturePresent(PF_FASTFAIL_AVAILABLE))
+        __fastfail(FAST_FAIL_FATAL_APP_EXIT);
+#  else
+    RaiseFailFastException(nullptr, nullptr, 0);
+#  endif
+
+    // Fallback
+    TerminateProcess(GetCurrentProcess(), STATUS_FATAL_APP_EXIT);
+
+    // Tell the compiler the application has stopped.
+    Q_UNREACHABLE_IMPL();
+#else // !Q_OS_WIN
     std::abort();
+#endif
 }
 
 
@@ -1870,9 +2065,7 @@ void qErrnoWarning(int code, const char *msg, ...)
     is not the default one.
 
     Example:
-    \code
-    QT_MESSAGE_PATTERN="[%{time yyyyMMdd h:mm:ss.zzz t} %{if-debug}D%{endif}%{if-info}I%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{file}:%{line} - %{message}"
-    \endcode
+    \snippet code/src_corelib_global_qlogging.cpp 0
 
     The default \a pattern is "%{if-category}%{category}: %{endif}%{message}".
 
@@ -1880,6 +2073,10 @@ void qErrnoWarning(int code, const char *msg, ...)
     environment variable; if both \l qSetMessagePattern() is called and QT_MESSAGE_PATTERN is
     set, the environment variable takes precedence.
 
+    \note The message pattern only applies to unstructured logging, such as the default
+    \c stderr output. Structured logging such as systemd will record the message as is,
+    along with as much structured information as can be captured.
+
     Custom message handlers can use qFormatLogMessage() to take \a pattern into account.
 
     \sa qInstallMessageHandler(), {Debugging Techniques}, {QLoggingCategory}
