@@ -41,7 +41,9 @@
 #include "qguiapplication.h"
 #include "qscreen.h"
 #include "qplatformintegration.h"
+#include "qplatformwindow.h"
 #include "private/qscreen_p.h"
+#include <private/qguiapplication_p.h>
 
 #include <QtCore/qdebug.h>
 
@@ -129,7 +131,7 @@ static inline qreal initialGlobalScaleFactor()
 
     Note that the functions in this file do not work with the OS scale factor
     directly and are limited to converting between device independent and native
-    pixels. The OS scale factor is accunted for by QWindow::devicePixelRatio()
+    pixels. The OS scale factor is accounted for by QWindow::devicePixelRatio()
     and similar functions.
 
     Configuration Examples:
@@ -168,7 +170,7 @@ static inline qreal initialGlobalScaleFactor()
 
     1) A global scale factor
         The QT_SCALE_FACTOR environment variable can be used to set
-        a global scale factor for all windows in the processs. This
+        a global scale factor for all windows in the process. This
         is useful for testing and debugging (you can simulate any
         devicePixelRatio without needing access to special hardware),
         and perhaps also for targeting a specific application to
@@ -204,7 +206,7 @@ static inline qreal initialGlobalScaleFactor()
         T fromNativePixels(T, QWindow*)
 
     The following classes in QtGui use native pixels, for the convenience of the
-    plataform plugins:
+    platform plugins:
         QPlatformWindow
         QPlatformScreen
         QWindowSystemInterface (API only - Events are in device independent pixels)
@@ -244,7 +246,8 @@ static inline bool usePixelDensity()
         return false;
     return QCoreApplication::testAttribute(Qt::AA_EnableHighDpiScaling)
         || (screenEnvValueOk && screenEnvValue > 0)
-        || (qEnvironmentVariableIsSet(legacyDevicePixelEnvVar) && qgetenv(legacyDevicePixelEnvVar).toLower() == "auto");
+        || (qEnvironmentVariableIsSet(legacyDevicePixelEnvVar) &&
+            qgetenv(legacyDevicePixelEnvVar).compare("auto", Qt::CaseInsensitive) == 0);
 }
 
 void QHighDpiScaling::initHighDpiScaling()
@@ -312,9 +315,12 @@ void QHighDpiScaling::updateHighDpiScaling()
     }
     m_active = m_globalScalingActive || m_screenFactorSet || m_pixelDensityScalingActive;
 
-    QPlatformScreen *primaryScreen = QGuiApplication::primaryScreen()->handle();
-    qreal sf = screenSubfactor(primaryScreen);
-    QDpi primaryDpi = primaryScreen->logicalDpi();
+    QScreen *primaryScreen = QGuiApplication::primaryScreen();
+    if (!primaryScreen)
+        return;
+    QPlatformScreen *platformScreen = primaryScreen->handle();
+    qreal sf = screenSubfactor(platformScreen);
+    QDpi primaryDpi = platformScreen->logicalDpi();
     m_logicalDpi = QDpi(primaryDpi.first / sf, primaryDpi.second / sf);
 }
 
@@ -372,6 +378,46 @@ QPoint QHighDpiScaling::mapPositionFromNative(const QPoint &pos, const QPlatform
     return (pos - topLeft) / scaleFactor + topLeft;
 }
 
+QPoint QHighDpiScaling::mapPositionToGlobal(const QPoint &pos, const QPoint &windowGlobalPosition, const QWindow *window)
+{
+    QPoint globalPosCandidate = pos + windowGlobalPosition;
+    if (QGuiApplicationPrivate::screen_list.size() <= 1)
+        return globalPosCandidate;
+
+    // The global position may be outside device independent screen geometry
+    // in cases where a window spans screens. Detect this case and map via
+    // native coordinates to the correct screen.
+    auto currentScreen = window->screen();
+    if (currentScreen && !currentScreen->geometry().contains(globalPosCandidate)) {
+        auto nativeGlobalPos = QHighDpi::toNativePixels(globalPosCandidate, currentScreen);
+        if (auto actualPlatformScreen = currentScreen->handle()->screenForPosition(nativeGlobalPos))
+            return QHighDpi::fromNativePixels(nativeGlobalPos, actualPlatformScreen->screen());
+    }
+
+    return globalPosCandidate;
+}
+
+QPoint QHighDpiScaling::mapPositionFromGlobal(const QPoint &pos, const QPoint &windowGlobalPosition, const QWindow *window)
+{
+    QPoint windowPosCandidate = pos - windowGlobalPosition;
+    if (QGuiApplicationPrivate::screen_list.size() <= 1)
+        return windowPosCandidate;
+
+    // Device independent global (screen) space may discontiguous when high-dpi scaling
+    // is active. This means that the normal subtracting of the window global position from the
+    // position-to-be-mapped may not work in cases where a window spans multiple screens.
+    // Map both positions to native global space (using the correct screens), subtract there,
+    // and then map the difference back using the scale factor for the window.
+    QScreen *posScreen = QGuiApplication::screenAt(pos);
+    if (posScreen && posScreen != window->screen()) {
+        QPoint nativePos = QHighDpi::toNativePixels(pos, posScreen);
+        QPoint windowNativePos = window->handle()->geometry().topLeft();
+        return QHighDpi::fromNativeLocalPosition(nativePos - windowNativePos, window);
+    }
+
+    return windowPosCandidate;
+}
+
 qreal QHighDpiScaling::screenSubfactor(const QPlatformScreen *screen)
 {
     qreal factor = qreal(1.0);
@@ -446,5 +492,13 @@ QPoint QHighDpiScaling::origin(const QPlatformScreen *platformScreen)
     return platformScreen->geometry().topLeft();
 }
 
+QPoint QHighDpiScaling::origin(const QWindow *window)
+{
+    if (window && window->isTopLevel() && window->screen())
+        return window->screen()->geometry().topLeft();
+
+    return QPoint(0, 0);
+}
+
 #endif //QT_NO_HIGHDPISCALING
 QT_END_NAMESPACE
