@@ -190,7 +190,6 @@ struct QTLWExtra {
     uint posIncludesFrame : 1;
     uint sizeAdjusted : 1;
     uint inTopLevelResize : 1;
-    uint inRepaint : 1;
     uint embedded : 1;
 
     // *************************** Platform specific values (bit fields first) **********
@@ -356,6 +355,8 @@ public:
     void createRecursively();
     void createWinId();
 
+    bool setScreenForPoint(const QPoint &pos);
+
     void createTLExtra();
     void createExtra();
     void deleteExtra();
@@ -382,10 +383,11 @@ public:
 
     void updateFont(const QFont &);
     inline void setFont_helper(const QFont &font) {
-        if (data.fnt.resolve() == font.resolve() && data.fnt == font)
+        if (directFontResolveMask == font.resolve() && data.fnt == font)
             return;
         updateFont(font);
     }
+    QFont localFont() const;
     void resolveFont();
     QFont naturalWidgetFont(uint inheritedMask) const;
 
@@ -481,6 +483,7 @@ public:
     void hide_sys();
     void hide_helper();
     void _q_showIfNotHidden();
+    void setVisible(bool);
 
     void setEnabled_helper(bool);
     static void adjustFlags(Qt::WindowFlags &flags, QWidget *w = 0);
@@ -728,6 +731,7 @@ public:
 #endif
 
     // Other variables.
+    uint directFontResolveMask;
     uint inheritedFontResolveMask;
     uint inheritedPaletteResolveMask;
     short leftmargin;
